/*
  RabidVWM LiteStep Module - version 1.0
	Hard-Coded Mouse Actions

  Copyright (C) 2002 Joshua Seagoe
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "RabidVWM.h"

void MouseSwitch(int x,int y);
void MouseDragPress(int x,int y);
void MouseDragMove(int x,int y);
void MouseDragRelease(int x,int y);
void MouseDragCancel(int x,int y);

/*
struct s_MouseAction {
	MouseProc *Press;
	MouseProc *Move;
	MouseProc *Release;
	MouseProc *Cancel;
};
*/
const s_MouseAction MouseTable[MOUSE_ACTION_COUNT] = {
	{ NULL, NULL, NULL, NULL }, // no action
	{ &MouseSwitch, NULL, NULL, NULL }, // switch to desk on click
	{ &MouseDragPress, &MouseDragMove, &MouseDragRelease, &MouseDragCancel }, // drag miniwindow
};

/************************************************************** Action 1 ***************/
// Switch desks

void MouseSwitch(int x,int y)
{
	TRACE(TRACE_STATUS,"MouseSwitch");
	int desk = DeskFromVWMPoint(x,y);
	if (desk>=0) SwitchDesk(desk);
}

/************************************************************** Action 2 ***************/
// Drag window
// snap on drop if control is held

struct s_MouseDragData {
	POINT MouseOff; /* offset from mouse to window (in screen coords) */ 
	s_WinData w;
};

void MouseDragPress(int x,int y)
{
	POINT pt;
	s_MouseDragData *mdd;
	s_WinData *pData;
	int mousecurrent = GlobalData.MouseCurrent;
	GlobalData.MouseCurrent = 0;

	TRACE(TRACE_STATUS,"MouseDragPress");

	pt.x = x; pt.y = y;

	/* find the topmost window at this point */ 
	if (!(GlobalData.pRemember = pData = WindowFromVWMPoint(&pt))) {
		TRACE(TRACE_STATUS,"No window to drag.");
		/* quit if none */ 
		return;
	}

	/* save the original window state */ 
	GlobalData.MouseActionData = mdd = new s_MouseDragData;
	CopyMemory(&mdd->w,pData,sizeof(s_WinData));

	ASSERTV(mdd!=NULL);

	/* get mouse offset */ 
	VWMToScreen(x,y,&mdd->MouseOff);
	mdd->MouseOff.x = mdd->w.Area.left - mdd->MouseOff.x;
	mdd->MouseOff.y = mdd->w.Area.top - mdd->MouseOff.y;

	GlobalData.MouseCurrent = mousecurrent;

	/* we want to track the mouse from now on */ 
	SetCapture(GlobalData.hMainWnd);
}

void MouseDragCancel(int x,int y)
{
	s_MouseDragData *mdd;
	SIZE amount;
	s_WinData *pData;

	mdd = (s_MouseDragData *)GlobalData.MouseActionData;
	pData = GlobalData.pRemember;

	TRACE(TRACE_STATUS,"MouseDragCancel");

	ASSERTV(pData!=NULL);

	/* no action until after we return */ 
	GlobalData.MouseCurrent = 0;

	if (mdd) {
		/* restore the original window's state, if it matches */ 
		if (pData->hWnd == mdd->w.hWnd) {

			amount.cx = mdd->w.Area.left - pData->Area.left;
			amount.cy = mdd->w.Area.top  - pData->Area.top;
			MoveWindowVWM(pData,&amount,GlobalData.CurrentDesk,mdd->w.HomeDesk,NULL);
		}

		/* free */ 
		delete mdd;
	}

	DCLEAR(GlobalData.MouseActionData);

	ReleaseCapture();
	UpdateWindowList();
	Repaint();
}

static bool MouseDragMoveWin(int x, int y, bool snap)
{
	SIZE amount;
	int desk;
	s_MouseDragData *mdd;
	s_WinData *pData;

	mdd = (s_MouseDragData *)GlobalData.MouseActionData;
	pData = GlobalData.pRemember;

	ASSERTV(mdd!=NULL);
	ASSERTV(pData!=NULL);

	/* if it should snap, reset and move by desk */ 
	if (snap) {

		desk = DeskFromVWMPoint(x,y);

		/* if the window is being moved back to home desk, make sure it updates */ 
		if (desk==mdd->w.HomeDesk && desk!=pData->HomeDesk)
			pData->bPosChanged = true;
		/* make MoveWindowToDesk think the window is back at */ 
		/* its original position (it doesn't check)          */ 
		/* in order to get original screen offset            */ 
		pData->Area = mdd->w.Area;
		SetHomeDesk(pData,mdd->w.HomeDesk);

		/* move it */ 
		return MoveWindowToDesk(pData,desk,NULL);

	} else {

		VWMToScreen(x,y,(POINT *)&amount);

		/* find top-left corner of window, as offset from last position */ 
		amount.cx += mdd->MouseOff.x - pData->Area.left;
		amount.cy += mdd->MouseOff.y - pData->Area.top;

		if (amount.cx==0 && amount.cy==0) return true;

		/* move the window */ 
		return MoveWindowVWM(pData,&amount,GlobalData.CurrentDesk,DeskFromVWMPoint(x,y),NULL);
	}
}

void MouseDragMove(int x,int y)
{
	s_MouseDragData *mdd;
	s_WinData *pData;

	mdd = (s_MouseDragData *)GlobalData.MouseActionData;
	pData = GlobalData.pRemember;

#ifndef NDEBUG
	if (mdd==NULL) {
		ASSERTV(mdd!=NULL);
		MouseDragCancel(x,y);
		return;
	}
#endif
	ASSERTV(pData!=NULL);
	ASSERTV(pData->hWnd == mdd->w.hWnd);

	MouseDragMoveWin(x,y,GlobalData.bSnapWindowOnDrag && !ControlKeyDown());

	/* update just this window to save trillions of cycles */ 
	UpdateRect(pData);
	if (pData->bPosChanged || pData->bSizeChanged)
		Repaint();
}

void MouseDragRelease(int x,int y)
{
	s_MouseDragData *mdd;
	s_WinData *pData;
	bool drag;

	mdd = (s_MouseDragData *)GlobalData.MouseActionData;
	pData = GlobalData.pRemember;

	TRACE(TRACE_STATUS,"MouseDragRelease");

	ASSERTV(mdd!=NULL);
	ASSERTV(pData!=NULL);

	/* no action until after we return */ 
	GlobalData.MouseCurrent = 0;

	drag = GlobalData.bSnapWindowOnDrag || IsZoomed(mdd->w.hWnd);
	if (ControlKeyDown()) drag = !drag;
	if (!MouseDragMoveWin(x,y,drag))
		MouseDragCancel(x,y);

	/* free the dragdata */ 

	delete mdd;
	DCLEAR(GlobalData.MouseActionData);

	/* make sure the home desk gets updated */ 
	pData->bPosChanged = true;

	UpdateWindowList();
	Repaint();
	ReleaseCapture();
}
