/*
  RabidVWM LiteStep Module - version 1.0
	Bitmap-Based Painting Functions

  Copyright (C) 2002 Joshua Seagoe
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "RabidVWM.h"

// Paint using bitmaps

void PaintVWM_Bitmap(HDC dc);
static void BeginPaintWnd_Bitmap(HDC dc,PixelBlock &,void *store);
static void PaintWnd_Bitmap(HDC dc,PixelBlock &,void *store,s_WinData *pData,RECT *rFull,RECT *rWnd);
static void PaintWndTitle_Bitmap(HDC dc,PixelBlock &,void *store,s_WinData *pData,RECT *rFull,RECT *rTitle);
static void EndPaintWnd_Bitmap(HDC dc,PixelBlock &,void *store);

const s_PaintWnd Paint_Bitmap = {
	&BeginPaintWnd_Bitmap,
	&PaintWnd_Bitmap,
	&PaintWndTitle_Bitmap,
	&EndPaintWnd_Bitmap
};

struct s_PaintWnd_Bitmap {
	HBITMAP bmWndOld;
	HDC dcWnd;

	HBITMAP bmTtlOld;
	HDC dcTitle;
};

void PaintVWM_Bitmap(HDC dc)
{
	HDC bdc;
	HBITMAP bmOld;
	COLORREF fg,bk;
	RECT vwmrect;

	bdc = CreateCompatibleDC(dc);
	bmOld = (HBITMAP)SelectObject(bdc,GlobalData.emBack.Bitmap);

	/* set fg and bk colors for monochrome bitmaps */ 
	fg = SetTextColor(dc,GlobalData.cBack.Border);
	bk = SetBkColor(dc,GlobalData.cBack.Flat);

	SetRect(&vwmrect, 0, 0, GlobalData.VWMSize.cx, GlobalData.VWMSize.cy);

	/* blit it */ 
	TransEdgeBlt(
		dc, &vwmrect,
		bdc, &GlobalData.emBack);

	/* clean up */ 
	SetTextColor(dc,fg);
	SetBkColor(dc,bk);

	SelectObject(bdc,bmOld);
	DeleteDC(bdc);
}

void PaintSelect_Bitmap(HDC dc,s_DeskData *pDesk,RECT *pArea)
{
	ASSERTV(GlobalData.emSel.Bitmap!=NULL);
	ASSERTV(pDesk!=NULL);
	ASSERTV(pArea!=NULL);

	HDC bdc;
	HBITMAP bmOld;
	COLORREF fg,bk;

	bdc = CreateCompatibleDC(dc);
	bmOld = (HBITMAP)SelectObject(bdc,GlobalData.emSel.Bitmap);

	/* set fg and bk colors for monochrome bitmaps */ 
	fg = SetTextColor(dc,GlobalData.cSelect.Border);
	bk = SetBkColor(dc,GlobalData.cSelect.Flat);

	TransEdgeBlt(dc, pArea, bdc, &GlobalData.emSel);

	/* clean up */ 
	SetTextColor(dc,fg);
	SetBkColor(dc,bk);

	SelectObject(bdc,bmOld);
	DeleteDC(bdc);
}

void BeginPaintWnd_Bitmap(HDC dc,PixelBlock &pb,void *store)
{
	s_PaintWnd_Bitmap *pwb;

	ASSERTV(store!=NULL);
	ASSERTV(PAINTWND_BUFFER_SIZE>=sizeof(s_PaintWnd_Bitmap));

	pwb = (s_PaintWnd_Bitmap *)store;

	pwb->dcWnd = CreateCompatibleDC(dc);
	CTRACE(TRACE_WARNING,!pwb->dcWnd,"BPW_B: Failed to create dcWnd.");
	pwb->bmWndOld = (HBITMAP)SelectObject(pwb->dcWnd,GlobalData.emWnd.Bitmap);

	if (GlobalData.bTitlebars) {
		pwb->dcTitle = CreateCompatibleDC(dc);
		CTRACE(TRACE_WARNING,!pwb->dcTitle,"BPW_B: Failed to create dcTitle.");
		pwb->bmTtlOld = (HBITMAP)SelectObject(pwb->dcTitle,GlobalData.emTitle.Bitmap);
	}
}

void PaintWnd_Bitmap(HDC dc,PixelBlock &pb,void *store,s_WinData *pData,RECT *rFull,RECT *rWnd)
{
	s_PaintWnd_Bitmap *pwb = (s_PaintWnd_Bitmap *)store;

	ASSERTV(store!=NULL);
	ASSERTV(pwb!=NULL);
	ASSERTV(pData!=NULL);
	ASSERTV(rFull!=NULL);
	ASSERTV(rWnd!=NULL);

	if (
		((GlobalData.bFlashState && pData->bFlash) || pData->bFlashOn)
		&& GlobalData.emFlash.Bitmap
		) {
		SelectObject(pwb->dcWnd,GlobalData.emFlash.Bitmap);
		TransEdgeBlt(dc, rFull, pwb->dcWnd, &GlobalData.emFlash);
		SelectObject(pwb->dcWnd,GlobalData.emWnd.Bitmap);
	} else {
		TransEdgeBlt(dc, rFull, pwb->dcWnd, &GlobalData.emWnd);
	}
}

void PaintWndTitle_Bitmap(HDC dc,PixelBlock &pb,void *store,s_WinData *pData,RECT *rFull,RECT *rTitle)
{
	s_PaintWnd_Bitmap *pwb = (s_PaintWnd_Bitmap *)store;

	ASSERTV(store!=NULL);
	ASSERTV(pwb!=NULL);
	ASSERTV(pData!=NULL);
	ASSERTV(rFull!=NULL);
	ASSERTV(rTitle!=NULL);

	if (
		((GlobalData.bFlashState && pData->bFlash) || pData->bFlashOn)
		&& GlobalData.emFlash.Bitmap
		&& GlobalData.emFlashTitle.Bitmap
		) {
		SelectObject(pwb->dcTitle,GlobalData.emFlashTitle.Bitmap);
		TransEdgeBlt(dc, rTitle, pwb->dcTitle, &GlobalData.emFlashTitle);
		SelectObject(pwb->dcTitle,GlobalData.emTitle.Bitmap);
	} else {
		TransEdgeBlt(dc, rTitle, pwb->dcTitle, &GlobalData.emTitle);
	}
}

void EndPaintWnd_Bitmap(HDC dc,PixelBlock &pb,void *store)
{
	s_PaintWnd_Bitmap *pwb = (s_PaintWnd_Bitmap *)store;
	DI;

	ASSERTV(pwb!=NULL);

	CTRACE(TRACE_WARNING,!i,"EPW_B: Failed to delete temp handles.");

	CI0;

	SelectObject(pwb->dcWnd,pwb->bmWndOld);
	CI DeleteDC(pwb->dcWnd);

	if (GlobalData.bTitlebars) {
		SelectObject(pwb->dcTitle,pwb->bmTtlOld);
		CI DeleteDC(pwb->dcTitle);
	}

	CTRACE(TRACE_WARNING,!i,"EPW_B: Failed to delete handles.");
}

